# -*- coding: utf-8 -*-
"""
celery cli handlers params module.
"""

import pyrin.configuration.services as config_services

from pyrin.task_queues.celery.cli.interface import CeleryCLIParamBase
from pyrin.cli.arguments import KeywordArgument, BooleanArgument, \
    PositionalArgument, CompositeKeywordArgument, CompositePositionalArgument, \
    JSONKeywordArgument


class LogFileParam(KeywordArgument, CeleryCLIParamBase):
    """
    log file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of LogFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('logfile', '--logfile', default=default)


class PIDFileParam(KeywordArgument, CeleryCLIParamBase):
    """
    pid file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of PIDFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('pidfile', '--pidfile', default=default)


class LogLevelParam(KeywordArgument, CeleryCLIParamBase):
    """
    log level param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of LogLevelParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('loglevel', '--loglevel', default=default)


class WorkerLogFileParam(LogFileParam):
    """
    worker log file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of WorkerLogFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_log_file` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_log_file')

        super().__init__(default=default)


class WorkerPIDFileParam(PIDFileParam):
    """
    worker pid file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of WorkerPIDFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_pid_file` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_pid_file')

        super().__init__(default=default)


class WorkerLogLevelParam(LogLevelParam):
    """
    worker log level param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of WorkerLogLevelParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_log_level` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_log_level')

        super().__init__(default=default)


class ConcurrencyParam(KeywordArgument, CeleryCLIParamBase):
    """
    concurrency param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ConcurrencyParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_concurrency` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_concurrency')

        super().__init__('concurrency', '--concurrency', default=default)


class HostnameParam(KeywordArgument, CeleryCLIParamBase):
    """
    hostname param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of HostnameParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_hostname` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_hostname')

        super().__init__('hostname', '--hostname', default=default)


class BeatParam(BooleanArgument, CeleryCLIParamBase):
    """
    beat param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of BeatParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('beat', '--beat', default=default)


class AutoScaleParam(CompositeKeywordArgument, CeleryCLIParamBase):
    """
    autoscale param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of AutoScaleParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_autoscale` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_autoscale')

        super().__init__('autoscale', '--autoscale', separator=',', default=default)


class QueuesParam(CompositeKeywordArgument, CeleryCLIParamBase):
    """
    queues param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of QueuesParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_queues` value
                               form `celery` config store if not provided.
        """

        super().__init__('queues', '--queues', separator=',', default=default)


class WorkerQueuesParam(QueuesParam):
    """
    worker queues param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of WorkerQueuesParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_queues` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_queues')

        super().__init__(default=default)


class PurgeParam(BooleanArgument, CeleryCLIParamBase):
    """
    purge param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of PurgeParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('purge', '--purge', default=default)


class OptimizationParam(KeywordArgument, CeleryCLIParamBase):
    """
    optimization param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of OptimizationParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_optimization` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'worker_optimization')

        super().__init__('optimization', '-O', default=default)


class BeatLogFileParam(LogFileParam):
    """
    beat log file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of BeatLogFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `beat_log_file` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'beat_log_file')

        super().__init__(default=default)


class BeatPIDFileParam(PIDFileParam):
    """
    beat pid file param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of BeatPIDFileParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `beat_pid_file` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'beat_pid_file')

        super().__init__(default=default)


class BeatLogLevelParam(LogLevelParam):
    """
    beat log level param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of BeatLogLevelParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `beat_log_level` value
                               form `celery` config store if not provided.
        """

        if default is None:
            default = config_services.get_active('celery', 'beat_log_level')

        super().__init__(default=default)


class TaskIDParam(PositionalArgument, CeleryCLIParamBase):
    """
    task id param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of TaskIDParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('task_id', index, default=default, **options)


class TracebackParam(BooleanArgument, CeleryCLIParamBase):
    """
    traceback param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of TracebackParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('traceback', '--traceback', default=default)


class TaskNameParam(KeywordArgument, CeleryCLIParamBase):
    """
    task name param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of TaskNameParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('task', '--task', default=default)


class InspectMethodParam(PositionalArgument, CeleryCLIParamBase):
    """
    inspect method param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of InspectMethodParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('method', index, default=default, **options)


class TimeoutParam(KeywordArgument, CeleryCLIParamBase):
    """
    timeout param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of TimeoutParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('timeout', '--timeout', default=default)


class DestinationParam(CompositeKeywordArgument, CeleryCLIParamBase):
    """
    destination param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of DestinationParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('destination', '--destination',
                         separator=',', default=default)


class JSONParam(BooleanArgument, CeleryCLIParamBase):
    """
    json param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of JSONParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('json', '--json', default=default)


class IncludeDefaultsParam(PositionalArgument, CeleryCLIParamBase):
    """
    include defaults param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of IncludeDefaultsParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('include_defaults', index, default=default, **options)


class SamplesCountParam(PositionalArgument, CeleryCLIParamBase):
    """
    samples count param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of SamplesCountParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('samples_count', index, default=default, **options)


class ObjectTypeParam(PositionalArgument, CeleryCLIParamBase):
    """
    object type param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of ObjectTypeParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('object_type', index, default=default, **options)


class CountParam(PositionalArgument, CeleryCLIParamBase):
    """
    count param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of CountParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('count', index, default=default, **options)


class MaxDepthParam(PositionalArgument, CeleryCLIParamBase):
    """
    max depth param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of MaxDepthParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('max_depth', index, default=default, **options)


class TaskIDListParam(CompositePositionalArgument, CeleryCLIParamBase):
    """
    task id list param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of TaskIDListParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('task_ids', index, default=default,
                         separator=' ', **options)


class AttributeListParam(CompositePositionalArgument, CeleryCLIParamBase):
    """
    attribute list param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of AttributeListParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('attributes', index, default=default,
                         separator=' ', **options)


class TaskNamePositionalParam(PositionalArgument, CeleryCLIParamBase):
    """
    task name positional param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of TaskNamePositionalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('task', index, default=default, **options)


class ArgsParam(JSONKeywordArgument, CeleryCLIParamBase):
    """
    args param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ArgsParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('args', '--args', default=default)


class KwargsParam(JSONKeywordArgument, CeleryCLIParamBase):
    """
    kwargs param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of KwargsParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('kwargs', '--kwargs', default=default)


class ETAParam(KeywordArgument, CeleryCLIParamBase):
    """
    eta param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ETAParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('eta', '--eta', default=default)


class CountdownParam(KeywordArgument, CeleryCLIParamBase):
    """
    countdown param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of CountdownParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('countdown', '--countdown', default=default)


class ExpiresParam(KeywordArgument, CeleryCLIParamBase):
    """
    expires param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ExpiresParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('expires', '--expires', default=default)


class SerializerParam(KeywordArgument, CeleryCLIParamBase):
    """
    serializer param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of SerializerParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('serializer', '--serializer', default=default)


class QueueParam(KeywordArgument, CeleryCLIParamBase):
    """
    queue param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of QueueParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('queue', '--queue', default=default)


class ExchangeParam(KeywordArgument, CeleryCLIParamBase):
    """
    exchange param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ExchangeParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('exchange', '--exchange', default=default)


class RoutingKeyParam(KeywordArgument, CeleryCLIParamBase):
    """
    routing key param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of RoutingKeyParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('routing_key', '--routing-key', default=default)


class ControlMethodParam(PositionalArgument, CeleryCLIParamBase):
    """
    control method param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of ControlMethodParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('method', index, default=default, **options)


class QueuePositionalParam(PositionalArgument, CeleryCLIParamBase):
    """
    queue positional param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of QueuePositionalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('queue', index, default=default, **options)


class ExchangePositionalParam(PositionalArgument, CeleryCLIParamBase):
    """
    exchange positional param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of ExchangePositionalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('exchange', index, default=default, **options)


class ExchangeTypePositionalParam(PositionalArgument, CeleryCLIParamBase):
    """
    exchange type positional param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of ExchangeTypePositionalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('type', index, default=default, **options)


class RoutingKeyPositionalParam(PositionalArgument, CeleryCLIParamBase):
    """
    routing key positional param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of RoutingKeyPositionalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('routing_key', index, default=default, **options)


class MaxScaleParam(PositionalArgument, CeleryCLIParamBase):
    """
    max scale param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of MaxScaleParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('max_scale', index, default=default, **options)


class MinScaleParam(PositionalArgument, CeleryCLIParamBase):
    """
    min scale param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of MinScaleParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('min_scale', index, default=default, **options)


class PoolResizeParam(PositionalArgument, CeleryCLIParamBase):
    """
    pool resize param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of PoolResizeParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('resize', index, default=default, **options)


class RateLimitParam(PositionalArgument, CeleryCLIParamBase):
    """
    rate limit param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of RateLimitParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('rate_limit', index, default=default, **options)


class SignalParam(PositionalArgument, CeleryCLIParamBase):
    """
    signal param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of SignalParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('signal', index, default=default, **options)


class SoftSecondsParam(PositionalArgument, CeleryCLIParamBase):
    """
    soft seconds param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of SoftSecondsParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('soft_secs', index, default=default, **options)


class HardSecondsParam(PositionalArgument, CeleryCLIParamBase):
    """
    hard seconds param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of HardSecondsParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('hard_secs', index, default=default, **options)


class CameraParam(KeywordArgument, CeleryCLIParamBase):
    """
    camera param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of CameraParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('camera', '--camera', default=default)


class FrequencyParam(KeywordArgument, CeleryCLIParamBase):
    """
    frequency param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of FrequencyParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('frequency', '--frequency', default=default)


class DumpParam(BooleanArgument, CeleryCLIParamBase):
    """
    dump param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of DumpParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('dump', '--dump', default=default)


class MaxRateParam(KeywordArgument, CeleryCLIParamBase):
    """
    max rate param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of MaxRateParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('maxrate', '--maxrate', default=default)


class IPythonParam(BooleanArgument, CeleryCLIParamBase):
    """
    ipython param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of IPythonParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('ipython', '--ipython', default=default)


class BPythonParam(BooleanArgument, CeleryCLIParamBase):
    """
    bpython param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of BPythonParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('bpython', '--bpython', default=default)


class PythonParam(BooleanArgument, CeleryCLIParamBase):
    """
    python param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of PythonParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('python', '--python', default=default)


class WithoutTasksParam(BooleanArgument, CeleryCLIParamBase):
    """
    without tasks param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of WithoutTasksParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('without_tasks', '--without-tasks', default=default)


class EventletParam(BooleanArgument, CeleryCLIParamBase):
    """
    eventlet param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of EventletParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('eventlet', '--eventlet', default=default)


class GeventParam(BooleanArgument, CeleryCLIParamBase):
    """
    gevent param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of GeventParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('gevent', '--gevent', default=default)


class TopicParam(PositionalArgument, CeleryCLIParamBase):
    """
    topic param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of TopicParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('topic', index, default=default, **options)


class ForceParam(BooleanArgument, CeleryCLIParamBase):
    """
    force param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ForceParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
        """

        super().__init__('force', '--force', default=default)


class ExcludeQueuesParam(CompositeKeywordArgument, CeleryCLIParamBase):
    """
    exclude queues param class.
    """

    def __init__(self, default=None):
        """
        initializes an instance of ExcludeQueuesParam.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to `worker_queues` value
                               form `celery` config store if not provided.
        """

        super().__init__('exclude_queues', '--exclude-queues',
                         separator=',', default=default)


class ActionParam(PositionalArgument, CeleryCLIParamBase):
    """
    action param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of ActionParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('action', index, default=default, **options)


class FilesParam(PositionalArgument, CeleryCLIParamBase):
    """
    files param class.
    """

    def __init__(self, index=None, default=None, **options):
        """
        initializes an instance of FilesParam.

        :param int index: zero based index of this param in cli command inputs.
                          defaults to 0 if not provided.

        :param object default: default value to be emitted to
                               cli if this param is not available.
                               if set to None, this param will not
                               be emitted at all.
                               defaults to None if not provided.

        :keyword bool validate_index: specifies that index of this argument
                                      must be validated. it could be helpful
                                      to set this to False when there are multiple
                                      arguments with the same index that will appear
                                      in different situations.
                                      defaults to True if not provided.
        """

        if index is None:
            index = 0

        super().__init__('files', index, default=default, **options)



# -*- coding: utf-8 -*-
"""
alembic interface module.
"""

import pyrin.configuration.services as config_services
import pyrin.database.migration.alembic.services as alembic_services

from pyrin.cli.base import CLIHandlerBase
from pyrin.cli.params import CLIParamBase


class AlembicCLIParamBase(CLIParamBase):
    """
    alembic cli param base class.

    all alembic cli param classes must be subclassed from this.
    """
    pass


class AlembicCLIHandlerBase(CLIHandlerBase):
    """
    alembic cli handler base class.

    all alembic cli handlers must be subclassed from this.
    """

    def __init__(self, name):
        """
        initializes an instance of AlembicCLIHandlerBase.

        :param str name: the handler name that should be registered
                         with. this name must be the exact name that
                         this handler must emmit to cli.
        """

        super().__init__(name)
        package = alembic_services.get_package_class()
        self._config_file_path = config_services.get_file_path(package.CONFIG_STORE_NAMES[0])

    def _get_common_cli_options(self):
        """
        gets the list of common cli options.

        :rtype: list
        """

        return ['alembic', '-c', self._config_file_path]



# -*- coding: utf-8 -*-
"""
deserializer exceptions module.
"""

from pyrin.core.exceptions import CoreException


class DeserializerManagerException(CoreException):
    """
    deserializer manager exception.
    """
    pass


class InvalidDeserializerTypeError(DeserializerManagerException):
    """
    invalid deserializer type error.
    """
    pass


class DuplicatedDeserializerError(DeserializerManagerException):
    """
    duplicated deserializer error.
    """
    pass



# -*- coding: utf-8 -*-
"""
celery services module.
"""

from pyrin.application.services import get_component
from pyrin.task_queues.celery import CeleryPackage


def get_current_app():
    """
    gets current celery application.

    :rtype: Celery
    """

    return get_component(CeleryPackage.COMPONENT_NAME).get_current_app()



# -*- coding: utf-8 -*-
"""
accounting models module.
"""

from sqlalchemy import Unicode, Integer, ForeignKey, CheckConstraint

from pyrin.database.model.base import CoreEntity
from pyrin.database.orm.sql.schema.base import CoreColumn
from pyrin.database.orm.types.custom import GUID


class AccountEntity(CoreEntity):

    _table = 'account'

    id = CoreColumn(name='account', type_=GUID, primary_key=True)
    owner = CoreColumn(name='owner', type_=Unicode)



# -*- coding: utf-8 -*-
"""
province component module.
"""

from pyrin.application.decorators import component
from pyrin.application.structs import Component

from demo.common.province import ProvincePackage
from demo.common.province.manager import ProvinceManager


@component(ProvincePackage.COMPONENT_NAME)
class ProvinceComponent(Component, ProvinceManager):
    """
    province component class.
    """
    pass



# -*- coding: utf-8 -*-
"""
api exceptions module.
"""

from pyrin.core.exceptions import CoreException


class APIManagerException(CoreException):
    """
    api manager exception.
    """
    pass


class InvalidAPIHookTypeError(APIManagerException):
    """
    invalid api hook type error.
    """
    pass



# -*- coding: utf-8 -*-
"""
core contexts module.
"""

from contextlib import AbstractContextManager


class ContextManagerBase(AbstractContextManager):
    """
    context manager base class.

    this class should be used as the base for all application context managers.
    """

    def _should_be_raised(self, internal_error, context_exception_type):
        """
        gets a value indicating that given internal error should be raised.

        considering the exception type that has been occurred inside context.

        :param Exception internal_error: exception instance that has been occurred
                                         inside the context manger itself.

        :param type[Exception] context_exception_type: the exception type that has been
                                                       occurred inside the context but
                                                       not in context manager itself.

        :rtype: bool
        """

        return context_exception_type is None or \
            type(internal_error) is not context_exception_type



# -*- coding: utf-8 -*-
"""
city api module.
"""

import time

from flask import url_for

import pyrin.globalization.datetime.services as datetime_services

from pyrin.api.router.decorators import api
from pyrin.caching.decorators import cached
from pyrin.caching.remote.decorators import memcached
from pyrin.core.enumerations import HTTPMethodEnum

import pyrin.processor.cors.services as cors_services
import demo.common.city.services as city_services
from pyrin.core.exceptions import CoreException
from pyrin.core.structs import DTO
from pyrin.security.session.services import get_current_request

#
# @api('/car/<slug>', authenticated=False)
# def test1(**options):
#     return 'test1'
#
#
# @api('/car/comments', authenticated=False, methods=HTTPMethodEnum.GET)
# def test2(**options):
#     preflight = cors_services.get_preflight_headers(['get', 'post'])
#     actual = cors_services.get_actual_headers()
#     actual.update(preflight)
#     return 'test2', actual
#
#
# @api('/cities/<int:city_id>', methods=HTTPMethodEnum.GET, authenticated=False)
# def get(city_id, **options):
#     """
#     gets the specified city.
#
#     :param int city_id: city id to get its info.
#
#     :raises CityNotFoundError: city not found error.
#
#     :returns: dict(int id,
#                    str name,
#                    int province_id: province id)
#
#     :rtype: dict
#     """
#
#     return city_services.get(city_id, **options)
#
#
# @api('/cities', methods=HTTPMethodEnum.DELETE, authenticated=False)
# def clear(**options):
#     return city_services.clear()
#
#
# @api('/cities', methods=HTTPMethodEnum.GET,
#      authenticated=False, no_cache=True, paged=True, indexed=True)
# def find(name, age, *, gender, level=None, **filters):
#     """
#     finds cities with given filters.
#
#     :keyword str name: city name.
#     :keyword int province_id: province id.
#
#     :returns: list[dict(int id,
#                         str name,
#                         int province_id: province id)]
#
#     :rtype: list
#     """
#
#     return city_services.find(name, **filters)
#
#
# @api('/cities', methods=HTTPMethodEnum.POST, authenticated=False)
# def create(**options):
#     """
#     creates a city.
#
#     :param str name: city name.
#     :param int province_id: province id.
#     """
#
#     return city_services.create(**options)
#
#
# @api('/template', methods=HTTPMethodEnum.GET, authenticated=False)
# def get_template(**options):
#     return """<form method="POST">
#                   Language: <input type="text" name="language"><br>
#                   Framework1: <input type="text" name="framework"><br>
#                   Framework2: <input type="text" name="framework"><br>
#                   <input type="submit" value="Submit"><br>
#               </form>"""
#
#
# @api('/template', methods=HTTPMethodEnum.POST, authenticated=False)
# def post_template(**options):
#     return options
#
#
# @api('/test', methods=(HTTPMethodEnum.POST, HTTPMethodEnum.GET), authenticated=False)
# def test(**options):
#     d = get_current_request()
#     return dict(hash=hash(d),
#                 hash_id=hash(d.request_id))
#
#
# @api('/check', methods=HTTPMethodEnum.GET, authenticated=False, request_limit=3, lifetime=20)
# def disappearing(**options):
#     return 'I will go away soon.'
#
#
# @api('/check', methods=HTTPMethodEnum.POST, authenticated=False)
# def disappearing2(**options):
#     return 'I will stay longer.'
#
#
# @api('/old-get', methods=HTTPMethodEnum.GET, authenticated=False, request_limit=1, lifetime=1)
# def old_get(**options):
#     return 'I will go away soon.'
#
#
# @api('/old-get', methods=HTTPMethodEnum.POST, authenticated=False, request_limit=1, lifetime=1)
# def old_post(**options):
#     return 'I will go away soon.'
#
#
# @api('/old-get', methods=(HTTPMethodEnum.GET, HTTPMethodEnum.POST), authenticated=False, replace=True)
# def new(**options):
#     return url_for('demo.common.city.api.old_gets')
#
#
# @api('/url-for', methods=(HTTPMethodEnum.GET, HTTPMethodEnum.POST), authenticated=False, replace=True)
# def url(**options):
#     return url_for('demo.common.city.no_way.api.disappearing', _external=True, city_id=100)
#
#
# @api('/public', methods=HTTPMethodEnum.GET, authenticated=False)
# def public(**options):
#     return 'I am public and here to stay.'
#
#
# @api('/public-temp', methods=HTTPMethodEnum.GET, authenticated=False, request_limit=5, lifetime=40)
# def public_temp(**options):
#     return 'I am public temp and here to go 40s.'
#
#
# @api('/protected', methods=HTTPMethodEnum.GET, authenticated=True)
# def protected(**options):
#     return 'I am protected and here to stay.'
#
#
# @api('/protected-temp', methods=HTTPMethodEnum.GET, authenticated=True, request_limit=3, lifetime=120)
# def protected_temp(**options):
#     return 'I am protected temp and here to go 120s.'
#
#
# @api('/fresh', methods=HTTPMethodEnum.GET, authenticated=True)
# def fresh(**options):
#     return 'I am fresh and here to stay.'
#
#
# @api('/fresh-temp', methods=HTTPMethodEnum.GET, authenticated=True, request_limit=3, lifetime=90)
# def fresh_temp(**options):
#     return 'I am fresh temp and here to go 90s.'
#
#
# @api('/cached', methods=HTTPMethodEnum.GET, authenticated=False)
# @cached(expire=1000000)
# def cached_method(name, age, **options):
#     return DTO(name=name, age=age, **options)
#
#
# @api('/memcached', methods=HTTPMethodEnum.GET, authenticated=False)
# @memcached(expire=0)
# def memcached_method(name, age, **options):
#     return DTO(date=time.time())
#
#
# @api('/time', methods=HTTPMethodEnum.GET, authenticated=False)
# def sentry_method(**options):
#     date = options.get('date')
#     now = datetime_services.now('europe/berlin')
#     options.update(type=str(type(date)), now=now)
#     return options



# -*- coding: utf-8 -*-
"""
celery cli handlers control module.
"""

from pyrin.task_queues.celery.cli.decorators import celery_cli_handler
from pyrin.task_queues.celery.cli.enumerations import CeleryCLIHandlersEnum
from pyrin.task_queues.celery.cli.interface import CeleryCLIHandlerBase
from pyrin.task_queues.celery.cli.handlers.params import ControlMethodParam, \
    QueuePositionalParam, ExchangePositionalParam, ExchangeTypePositionalParam, \
    RoutingKeyPositionalParam, PoolResizeParam, MinScaleParam, MaxScaleParam, \
    RateLimitParam, SignalParam, SoftSecondsParam, HardSecondsParam, DestinationParam, \
    JSONParam, TimeoutParam, TaskNamePositionalParam, TaskIDListParam


@celery_cli_handler()
class ControlCLIHandler(CeleryCLIHandlerBase):
    """
    control cli handler class.
    """

    def __init__(self):
        """
        initializes an instance of ControlCLIHandler.
        """

        super().__init__(CeleryCLIHandlersEnum.CONTROL)

    def _inject_params(self, params):
        """
        injects all the params of current handler into given list.

        :param list[CLIParamBase] params: list of all params.
        """

        params.extend([ControlMethodParam(index=0),
                       QueuePositionalParam(index=1),
                       MaxScaleParam(index=1, validate_index=False),
                       PoolResizeParam(index=1, validate_index=False),
                       TaskNamePositionalParam(index=1, validate_index=False),
                       TaskIDListParam(index=1, validate_index=False),
                       SignalParam(index=1, validate_index=False),
                       ExchangePositionalParam(index=2),
                       MinScaleParam(index=2, validate_index=False),
                       RateLimitParam(index=2, validate_index=False),
                       TaskIDListParam(index=2, validate_index=False),
                       SoftSecondsParam(index=2, validate_index=False),
                       ExchangeTypePositionalParam(index=3),
                       HardSecondsParam(index=3, validate_index=False),
                       RoutingKeyPositionalParam(index=4),
                       TimeoutParam(), DestinationParam(), JSONParam()])

        return super()._inject_params(params)



# -*- coding: utf-8 -*-
"""
api component module.
"""

from pyrin.application.decorators import component
from pyrin.api import APIPackage
from pyrin.api.manager import APIManager
from pyrin.application.structs import Component


@component(APIPackage.COMPONENT_NAME)
class APIComponent(Component, APIManager):
    """
    api component class.
    """
    pass



# -*- coding: utf-8 -*-
"""
caching remote decorators module.
"""

from functools import update_wrapper

import pyrin.caching.services as caching_services


def memcached(*old_method, **options):
    """
    decorator to convert a method or function into a lazy one.

    note that this cache type supports expire time and will consider method inputs
    in caching. the result will be calculated once and then it will be cached.
    each result will be cached using a tuple of class type, method name, inputs,
    current user and component key as a key in the cache.

    that this decorator could be used on both instance or class level methods and
    properties or stand-alone functions.

    to be able to use this decorator you must install memcached client dependency
    using `pip install pyrin[memcached]` and also remove
    `pyrin.caching.remote.handlers.memcached` from `ignored_modules` of
    `packaging.ini` file.

    :param function | property old_method: the original decorated method or function.

    :keyword bool consider_user: specifies that current user must be included in
                                 key generation. if not provided, it will be get
                                 from `caching` config store.

    :keyword int expire: expire time for given key in seconds.
                         if not provided, it will be get from `caching`
                         config store.

    :returns: method or function result.
    """

    def decorator(method):
        """
        decorates the given method or function and makes it a lazy one.

        :param function | property method: decorated method or function.

        :returns: method or function result.
        """

        def wrapper(*args, **kwargs):
            """
            decorates the given method or function and makes it a lazy one.

            :param object args: function positional arguments.
            :param object kwargs: function keyword arguments.

            :returns: method or function result.
            """

            result = caching_services.try_get('memcached', method, args,
                                              kwargs, **options)
            if result is not None:
                return result

            result = method(*args, **kwargs)
            caching_services.try_set('memcached', result, method,
                                     args, kwargs, **options)
            return result

        return update_wrapper(wrapper, method)

    if len(old_method) > 0:
        return decorator(old_method[0])

    return decorator


def redis(*old_method, **options):
    """
    decorator to convert a method or function into a lazy one.

    note that this cache type supports expire time and will consider method inputs
    in caching. the result will be calculated once and then it will be cached.
    each result will be cached using a tuple of class type, method name, inputs,
    current user and component key as a key in the cache.

    that this decorator could be used on both instance or class level methods and
    properties or stand-alone functions.

    to be able to use this decorator you must install redis client dependency
    using `pip install pyrin[redis]` and also remove
    `pyrin.caching.remote.handlers.redis` from `ignored_modules` of
    `packaging.ini` file.

    :param function | property old_method: the original decorated method or function.

    :keyword bool consider_user: specifies that current user must be included in
                                 key generation. if not provided, it will be get
                                 from `caching` config store.

    :keyword int expire: expire time for given key in milliseconds.
                         if not provided, it will be get from `caching`
                         config store.

    :returns: method or function result.
    """

    def decorator(method):
        """
        decorates the given method or function and makes it a lazy one.

        :param function | property method: decorated method or function.

        :returns: method or function result.
        """

        def wrapper(*args, **kwargs):
            """
            decorates the given method or function and makes it a lazy one.

            :param object args: function positional arguments.
            :param object kwargs: function keyword arguments.

            :returns: method or function result.
            """

            result = caching_services.try_get('redis', method, args,
                                              kwargs, **options)
            if result is not None:
                return result

            result = method(*args, **kwargs)
            caching_services.try_set('redis', result, method,
                                     args, kwargs, **options)
            return result

        return update_wrapper(wrapper, method)

    if len(old_method) > 0:
        return decorator(old_method[0])

    return decorator



# -*- coding: utf-8 -*-
"""
alembic component module.
"""

from pyrin.application.decorators import component
from pyrin.database.migration.alembic import AlembicPackage
from pyrin.database.migration.alembic.manager import AlembicManager
from pyrin.application.structs import Component


@component(AlembicPackage.COMPONENT_NAME)
class AlembicComponent(Component, AlembicManager):
    """
    alembic component class.
    """
    pass



# -*- coding: utf-8 -*-
"""
application enumerations module.
"""

from pyrin.core.enumerations import CoreEnum


class ApplicationStatusEnum(CoreEnum):
    """
    application status enum.
    """

    INITIALIZING = 'Initializing'
    LOADING = 'Loading'
    READY = 'Ready'
    RUNNING = 'Running'
    TERMINATED = 'Terminated'



# -*- coding: utf-8 -*-
"""
authentication manager module.
"""

from abc import abstractmethod

import pyrin.security.token.services as token_services
import pyrin.security.session.services as session_services

from pyrin.core.globals import _
from pyrin.core.structs import Manager
from pyrin.security.authentication import AuthenticationPackage
from pyrin.security.enumerations import TokenTypeEnum
from pyrin.core.exceptions import CoreNotImplementedError
from pyrin.security.authentication.exceptions import AuthenticationFailedError, \
    AccessTokenRequiredError, InvalidPayloadDataError


class AuthenticationManager(Manager):
    """
    authentication manager class.

    this class is intended to be an interface for top level
    application's authentication package.
    """

    package_class = AuthenticationPackage

    def authenticate(self, client_request, **options):
        """
        authenticates given request and pushes the authenticated data into request context.

        if authentication fails, authenticated data will not be pushed into request context.

        :param CoreRequest client_request: request to be authenticated.

        :raises AuthenticationFailedError: authentication failed error.
        :raises InvalidPayloadDataError: invalid payload data error.
        :raises AccessTokenRequiredError: access token required error.
        """

        token = self._extract_token(client_request)
        if token in (None, ''):
            return

        self._authenticate(token, **options)

    def _authenticate(self, token, **options):
        """
        authenticates given token and pushes the authenticated data into request context.

        if authentication fails, authenticated data will not be pushed into request context.

        :param str token: token to be authenticated.

        :raises AuthenticationFailedError: authentication failed error.
        :raises InvalidPayloadDataError: invalid payload data error.
        :raises AccessTokenRequiredError: access token required error.
        """

        try:
            payload = token_services.get_payload(token, **options)
            header = token_services.get_unverified_header(token, **options)
            self._validate(header, payload, **options)
            self._push_data(header, payload, **options)

        except AuthenticationFailedError:
            raise
        except Exception as error:
            raise AuthenticationFailedError(error) from error

    def _extract_token(self, client_request):
        """
        extracts token from request header if available.

        :param CoreRequest client_request: request object.

        :returns: token
        :rtype: str
        """

        return client_request.authorization

    def _push_data(self, header, payload, **options):
        """
        pushes the required data into current request from input values.

        :param dict header: token header data.
        :param dict payload: payload data of authenticated token.
        """

        session_services.add_request_context('token_header', header)
        session_services.add_request_context('token_payload', payload)

        self._push_custom_data(header, payload, **options)

    @abstractmethod
    def _push_custom_data(self, header, payload, **options):
        """
        pushes the custom data into current request from input values.

        :param dict header: token header data.
        :param dict payload: payload data of authenticated token.

        :raises CoreNotImplementedError: core not implemented error.
        """

        raise CoreNotImplementedError()

    def _push_component_custom_key(self, value):
        """
        pushes the provided value as component custom key into request.

        this method could be called in subclasses of this class in
        `_push_custom_data()` method if needed.

        :param object value: value to be pushed as component custom key.

        :raises InvalidComponentCustomKeyError: invalid component custom key error.
        """

        session_services.set_component_custom_key(value)

    def _validate(self, header, payload, **options):
        """
        validates the given inputs.

        an error will be raised if validation fails.

        :param dict header: token header data.
        :param dict payload: payload data to be validated.

        :raises InvalidPayloadDataError: invalid payload data error.
        :raises AccessTokenRequiredError: access token required error.
        """

        if payload is None:
            raise InvalidPayloadDataError(_('Payload data could not be None.'))

        token_type = payload.get('type', None)
        if token_type != TokenTypeEnum.ACCESS:
            raise AccessTokenRequiredError(_('Access token is required for authentication.'))

        self._validate_custom(header, payload, **options)

    @abstractmethod
    def _validate_custom(self, header, payload, **options):
        """
        validates the given inputs for custom attributes.

        an error will be raised if validation fails.

        :param dict header: token header data.
        :param dict payload: payload data to be validated.

        :raises CoreNotImplementedError: core not implemented error.
        :raises AuthenticationFailedError: authentication failed error.
        """

        raise CoreNotImplementedError()



# -*- coding: utf-8 -*-
"""
sequence services module.
"""

from pyrin.application.services import get_component
from pyrin.database.sequence import SequencePackage


def get_next_value(name):
    """
    gets the next value of given sequence.

    :param str name: sequence name to get its next value.

    :rtype: int
    """

    return get_component(SequencePackage.COMPONENT_NAME).get_next_value(name)



# -*- coding: utf-8 -*-
"""
alembic manager module.
"""

import pyrin.template.services as template_services

from pyrin.cli.mixin.handler import CLIMixin
from pyrin.core.structs import Manager
from pyrin.database.migration.alembic import AlembicPackage
from pyrin.database.migration.alembic.enumerations import AlembicCLIHandlersEnum
from pyrin.database.migration.alembic.interface import AlembicCLIHandlerBase


class AlembicManager(Manager, CLIMixin):
    """
    alembic manager class.
    """

    _cli_handler_type = AlembicCLIHandlerBase
    package_class = AlembicPackage

    def enable(self):
        """
        enables migrations for the application.
        """

        return template_services.create(AlembicCLIHandlersEnum.ENABLE)
